@@ -7,8 +7,9 @@ from operator import methodcaller
 from typing import TYPE_CHECKING, Any, ClassVar, Literal
 
 from narwhals._compliant import EagerGroupBy
+from narwhals._exceptions import issue_warning
 from narwhals._expression_parsing import evaluate_output_names_and_aliases
-from narwhals._utils import find_stacklevel
+from narwhals._utils import zip_strict
 from narwhals.dependencies import is_pandas_like_dataframe
 
 if TYPE_CHECKING:
@@ -38,6 +39,7 @@ NativeAggregation: TypeAlias = Literal[
     "mean",
     "median",
     "min",
+    "mode",
     "nunique",
     "prod",
     "quantile",
@@ -106,8 +108,43 @@ class AggExpr:
         """Evaluate the wrapped expression as a group_by operation."""
         result: pd.DataFrame | pd.Series[Any]
         names = self.output_names
-        if self.is_len() and self.is_anonymous():
+        if self.is_len() and self.is_top_level_function():
             result = group_by._grouped.size()
+        elif self.is_len():
+            result_single = group_by._grouped.size()
+            ns = group_by.compliant.__narwhals_namespace__()
+            result = ns._concat_horizontal(
+                [ns.from_native(result_single).alias(name).native for name in names]
+            )
+        elif self.is_mode():
+            compliant = group_by.compliant
+            if (keep := self.kwargs.get("keep")) != "any":  # pragma: no cover
+                msg = (
+                    f"`Expr.mode(keep='{keep}')` is not implemented in group by context for "
+                    f"backend {compliant._implementation}\n\n"
+                    "Hint: Use `nw.col(...).mode(keep='any')` instead."
+                )
+                raise NotImplementedError(msg)
+
+            cols = list(names)
+            native = compliant.native
+            keys, kwargs = group_by._keys, group_by._kwargs
+
+            # Implementation based on the following suggestion:
+            # https://github.com/pandas-dev/pandas/issues/19254#issuecomment-778661578
+            ns = compliant.__narwhals_namespace__()
+            result = ns._concat_horizontal(
+                [
+                    native.groupby([*keys, col], **kwargs)
+                    .size()
+                    .sort_values(ascending=False)
+                    .reset_index(col)
+                    .groupby(keys, **kwargs)[col]
+                    .head(1)
+                    .sort_index()
+                    for col in cols
+                ]
+            )
         else:
             select = names[0] if len(names) == 1 else list(names)
             result = self.native_agg()(group_by._grouped[select])
@@ -120,7 +157,11 @@ class AggExpr:
     def is_len(self) -> bool:
         return self.leaf_name == "len"
 
-    def is_anonymous(self) -> bool:
+    def is_mode(self) -> bool:
+        return self.leaf_name == "mode"
+
+    def is_top_level_function(self) -> bool:
+        # e.g. `nw.len()`.
         return self.expr._depth == 0
 
     @property
@@ -150,6 +191,7 @@ class PandasLikeGroupBy(
         "median": "median",
         "max": "max",
         "min": "min",
+        "mode": "mode",
         "std": "std",
         "var": "var",
         "len": "size",
@@ -168,6 +210,9 @@ class PandasLikeGroupBy(
     _output_key_names: list[str]
     """Stores the **original** version of group keys."""
 
+    _kwargs: Mapping[str, bool]
+    """Stores keyword arguments for `DataFrame.groupby` other than `by`."""
+
     @property
     def exclude(self) -> tuple[str, ...]:
         """Group keys to ignore when expanding multi-output aggregations."""
@@ -192,13 +237,14 @@ class PandasLikeGroupBy(
         native = self.compliant.native
         if set(native.index.names).intersection(self.compliant.columns):
             native = native.reset_index(drop=True)
-        self._grouped: NativeGroupBy = native.groupby(
-            self._keys.copy(),
-            sort=False,
-            as_index=True,
-            dropna=drop_null_keys,
-            observed=True,
-        )
+
+        self._kwargs = {
+            "sort": False,
+            "as_index": True,
+            "dropna": drop_null_keys,
+            "observed": True,
+        }
+        self._grouped: NativeGroupBy = native.groupby(self._keys.copy(), **self._kwargs)
 
     def agg(self, *exprs: PandasLikeExpr) -> PandasLikeDataFrame:
         all_aggs_are_simple = True
@@ -263,8 +309,7 @@ class PandasLikeGroupBy(
         apply = self._grouped.apply
         if impl.is_pandas() and impl._backend_version() >= (2, 2):
             return apply(func, include_groups=False)  # type: ignore[call-overload]
-        else:  # pragma: no cover
-            return apply(func)
+        return apply(func)  # pragma: no cover
 
     def _apply_exprs_function(self, exprs: Iterable[PandasLikeExpr]) -> NativeApply:
         ns = self.compliant.__narwhals_namespace__()
@@ -277,7 +322,7 @@ class PandasLikeGroupBy(
                 for expr in exprs
                 for keys in expr(compliant)
             )
-            out_group, out_names = zip(*results) if results else ([], [])
+            out_group, out_names = zip_strict(*results) if results else ([], [])
             return into_series(out_group, index=out_names, context=ns).native
 
         return fn
@@ -310,12 +355,11 @@ def empty_results_error() -> ValueError:
 
 
 def warn_complex_group_by() -> None:
-    warnings.warn(
+    issue_warning(
         "Found complex group-by expression, which can't be expressed efficiently with the "
         "pandas API. If you can, please rewrite your query such that group-by aggregations "
         "are simple (e.g. mean, std, min, max, ...). \n\n"
         "Please see: "
         "https://narwhals-dev.github.io/narwhals/concepts/improve_group_by_operation/",
         UserWarning,
-        stacklevel=find_stacklevel(),
     )
