@@ -1,12 +1,13 @@
 from __future__ import annotations
 
 import functools
+import operator
 import re
-from typing import TYPE_CHECKING, Any, Callable, Literal, TypeVar
+from typing import TYPE_CHECKING, Any, Callable, Literal, TypeVar, cast
 
 import pandas as pd
 
-from narwhals._compliant.series import EagerSeriesNamespace
+from narwhals._compliant import EagerSeriesNamespace
 from narwhals._constants import (
     MS_PER_SECOND,
     NS_PER_MICROSECOND,
@@ -15,22 +16,26 @@ from narwhals._constants import (
     SECONDS_PER_DAY,
     US_PER_SECOND,
 )
+from narwhals._exceptions import issue_warning
 from narwhals._utils import (
     Implementation,
     Version,
     _DeferredIterable,
     check_columns_exist,
     isinstance_or_issubclass,
+    requires,
 )
 from narwhals.exceptions import ShapeError
 
 if TYPE_CHECKING:
+    from collections.abc import Iterable, Iterator, Mapping
     from types import ModuleType
 
     from pandas._typing import Dtype as PandasDtype
     from pandas.core.dtypes.dtypes import BaseMaskedDtype
-    from typing_extensions import TypeIs
+    from typing_extensions import TypeAlias, TypeIs
 
+    from narwhals._duration import IntervalUnit
     from narwhals._pandas_like.expr import PandasLikeExpr
     from narwhals._pandas_like.series import PandasLikeSeries
     from narwhals._pandas_like.typing import (
@@ -42,6 +47,10 @@ if TYPE_CHECKING:
     from narwhals.typing import DTypeBackend, IntoDType, TimeUnit, _1DArray
 
     ExprT = TypeVar("ExprT", bound=PandasLikeExpr)
+    UnitCurrent: TypeAlias = TimeUnit
+    UnitTarget: TypeAlias = TimeUnit
+    BinOpBroadcast: TypeAlias = Callable[[Any, int], Any]
+    IntoRhs: TypeAlias = int
 
 
 PANDAS_LIKE_IMPLEMENTATION = {
@@ -95,7 +104,42 @@ PA_DURATION_RGX = r"""^
 $"""
 PATTERN_PA_DURATION = re.compile(PA_DURATION_RGX, re.VERBOSE)
 
-UNIT_DICT = {"d": "D", "m": "min"}
+NativeIntervalUnit: TypeAlias = Literal[
+    "year",
+    "quarter",
+    "month",
+    "week",
+    "day",
+    "hour",
+    "minute",
+    "second",
+    "millisecond",
+    "microsecond",
+    "nanosecond",
+]
+ALIAS_DICT = {"d": "D", "m": "min"}
+UNITS_DICT: Mapping[IntervalUnit, NativeIntervalUnit] = {
+    "y": "year",
+    "q": "quarter",
+    "mo": "month",
+    "d": "day",
+    "h": "hour",
+    "m": "minute",
+    "s": "second",
+    "ms": "millisecond",
+    "us": "microsecond",
+    "ns": "nanosecond",
+}
+
+PANDAS_VERSION = Implementation.PANDAS._backend_version()
+"""Static backend version for `pandas`.
+
+Always available if we reached here, due to a module-level import.
+"""
+
+
+def is_pandas_or_modin(implementation: Implementation) -> bool:
+    return implementation in {Implementation.PANDAS, Implementation.MODIN}
 
 
 def align_and_extract_native(
@@ -119,12 +163,7 @@ def align_and_extract_native(
         if rhs.native.index is not lhs_index:
             return (
                 lhs.native,
-                set_index(
-                    rhs.native,
-                    lhs_index,
-                    implementation=rhs._implementation,
-                    backend_version=rhs._backend_version,
-                ),
+                set_index(rhs.native, lhs_index, implementation=rhs._implementation),
             )
         return (lhs.native, rhs.native)
 
@@ -136,11 +175,7 @@ def align_and_extract_native(
 
 
 def set_index(
-    obj: NativeNDFrameT,
-    index: Any,
-    *,
-    implementation: Implementation,
-    backend_version: tuple[int, ...],
+    obj: NativeNDFrameT, index: Any, *, implementation: Implementation
 ) -> NativeNDFrameT:
     """Wrapper around pandas' set_axis to set object index.
 
@@ -156,26 +191,23 @@ def set_index(
         obj.index = index
         return obj
     if implementation is Implementation.PANDAS and (
-        (1, 5) <= backend_version < (3,)
+        (1, 5) <= implementation._backend_version() < (3,)
     ):  # pragma: no cover
         return obj.set_axis(index, axis=0, copy=False)
-    else:  # pragma: no cover
-        return obj.set_axis(index, axis=0)
+    return obj.set_axis(index, axis=0)  # pragma: no cover
 
 
 def rename(
-    obj: NativeNDFrameT,
-    *args: Any,
-    implementation: Implementation,
-    backend_version: tuple[int, ...],
-    **kwargs: Any,
+    obj: NativeNDFrameT, *args: Any, implementation: Implementation, **kwargs: Any
 ) -> NativeNDFrameT:
     """Wrapper around pandas' rename so that we can set `copy` based on implementation/version."""
     if implementation is Implementation.PANDAS and (
-        backend_version >= (3,)
+        implementation._backend_version() >= (3,)
     ):  # pragma: no cover
-        return obj.rename(*args, **kwargs, inplace=False)
-    return obj.rename(*args, **kwargs, copy=False, inplace=False)
+        result = obj.rename(*args, **kwargs, inplace=False)
+    else:
+        result = obj.rename(*args, **kwargs, copy=False, inplace=False)
+    return cast("NativeNDFrameT", result)  # type: ignore[redundant-cast]
 
 
 @functools.lru_cache(maxsize=16)
@@ -215,7 +247,15 @@ def non_object_native_to_narwhals_dtype(native_dtype: Any, version: Version) ->
         "float[pyarrow]",
     }:
         return dtypes.Float32()
-    if dtype in {"string", "string[python]", "string[pyarrow]", "large_string[pyarrow]"}:
+    if dtype in {
+        # "there is no problem which can't be solved by adding an extra string type" pandas
+        "string",
+        "string[python]",
+        "string[pyarrow]",
+        "string[pyarrow_numpy]",
+        "large_string[pyarrow]",
+        "str",
+    }:
         return dtypes.String()
     if dtype in {"bool", "boolean", "boolean[pyarrow]", "bool[pyarrow]"}:
         return dtypes.Boolean()
@@ -246,7 +286,7 @@ def non_object_native_to_narwhals_dtype(native_dtype: Any, version: Version) ->
 
 
 def object_native_to_narwhals_dtype(
-    series: PandasLikeSeries, version: Version, implementation: Implementation
+    series: PandasLikeSeries | None, version: Version, implementation: Implementation
 ) -> DType:
     dtypes = version.dtypes
     if implementation is Implementation.CUDF:
@@ -254,14 +294,15 @@ def object_native_to_narwhals_dtype(
         # objects, so we can just return String.
         return dtypes.String()
 
+    infer = pd.api.types.infer_dtype
     # Arbitrary limit of 100 elements to use to sniff dtype.
-    inferred_dtype = pd.api.types.infer_dtype(series.head(100), skipna=True)
+    inferred_dtype = "empty" if series is None else infer(series.head(100), skipna=True)
     if inferred_dtype == "string":
         return dtypes.String()
     if inferred_dtype == "empty" and version is not Version.V1:
         # Default to String for empty Series.
         return dtypes.String()
-    elif inferred_dtype == "empty":
+    if inferred_dtype == "empty":
         # But preserve returning Object in V1.
         return dtypes.Object()
     return dtypes.Object()
@@ -296,7 +337,11 @@ def _cudf_categorical_to_list(
 
 
 def native_to_narwhals_dtype(
-    native_dtype: Any, version: Version, implementation: Implementation
+    native_dtype: Any,
+    version: Version,
+    implementation: Implementation,
+    *,
+    allow_object: bool = False,
 ) -> DType:
     str_dtype = str(native_dtype)
 
@@ -317,10 +362,12 @@ def native_to_narwhals_dtype(
         )
     if str_dtype != "object":
         return non_object_native_to_narwhals_dtype(native_dtype, version)
-    elif implementation is Implementation.DASK:
+    if implementation is Implementation.DASK:
         # Per conversations with their maintainers, they don't support arbitrary
         # objects, so we can just return String.
         return version.dtypes.String()
+    if allow_object:
+        return object_native_to_narwhals_dtype(None, version, implementation)
     msg = (
         "Unreachable code, object dtype should be handled separately"  # pragma: no cover
     )
@@ -367,107 +414,102 @@ def get_dtype_backend(dtype: Any, implementation: Implementation) -> DTypeBacken
     return "numpy_nullable" if is_dtype_numpy_nullable(dtype) else None
 
 
+# NOTE: Use this to avoid annotating inline
+def iter_dtype_backends(
+    dtypes: Iterable[Any], implementation: Implementation
+) -> Iterator[DTypeBackend]:
+    """Yield a `DTypeBackend` per-dtype.
+
+    Matches pandas' `dtype_backend` argument in `convert_dtypes`.
+    """
+    return (get_dtype_backend(dtype, implementation) for dtype in dtypes)
+
+
 @functools.lru_cache(maxsize=16)
 def is_dtype_pyarrow(dtype: Any) -> TypeIs[pd.ArrowDtype]:
     return hasattr(pd, "ArrowDtype") and isinstance(dtype, pd.ArrowDtype)
 
 
-def narwhals_to_native_dtype(  # noqa: C901, PLR0912, PLR0915
+dtypes = Version.MAIN.dtypes
+NW_TO_PD_DTYPES_INVARIANT: Mapping[type[DType], str] = {
+    # TODO(Unassigned): is there no pyarrow-backed categorical?
+    # or at least, convert_dtypes(dtype_backend='pyarrow') doesn't
+    # convert to it?
+    dtypes.Categorical: "category",
+    dtypes.Object: "object",
+}
+NW_TO_PD_DTYPES_BACKEND: Mapping[type[DType], Mapping[DTypeBackend, str | type[Any]]] = {
+    dtypes.Float64: {
+        "pyarrow": "Float64[pyarrow]",
+        "numpy_nullable": "Float64",
+        None: "float64",
+    },
+    dtypes.Float32: {
+        "pyarrow": "Float32[pyarrow]",
+        "numpy_nullable": "Float32",
+        None: "float32",
+    },
+    dtypes.Int64: {"pyarrow": "Int64[pyarrow]", "numpy_nullable": "Int64", None: "int64"},
+    dtypes.Int32: {"pyarrow": "Int32[pyarrow]", "numpy_nullable": "Int32", None: "int32"},
+    dtypes.Int16: {"pyarrow": "Int16[pyarrow]", "numpy_nullable": "Int16", None: "int16"},
+    dtypes.Int8: {"pyarrow": "Int8[pyarrow]", "numpy_nullable": "Int8", None: "int8"},
+    dtypes.UInt64: {
+        "pyarrow": "UInt64[pyarrow]",
+        "numpy_nullable": "UInt64",
+        None: "uint64",
+    },
+    dtypes.UInt32: {
+        "pyarrow": "UInt32[pyarrow]",
+        "numpy_nullable": "UInt32",
+        None: "uint32",
+    },
+    dtypes.UInt16: {
+        "pyarrow": "UInt16[pyarrow]",
+        "numpy_nullable": "UInt16",
+        None: "uint16",
+    },
+    dtypes.UInt8: {"pyarrow": "UInt8[pyarrow]", "numpy_nullable": "UInt8", None: "uint8"},
+    dtypes.String: {"pyarrow": "string[pyarrow]", "numpy_nullable": "string", None: str},
+    dtypes.Boolean: {
+        "pyarrow": "boolean[pyarrow]",
+        "numpy_nullable": "boolean",
+        None: "bool",
+    },
+}
+UNSUPPORTED_DTYPES = (dtypes.Decimal,)
+
+
+def narwhals_to_native_dtype(  # noqa: C901, PLR0912
     dtype: IntoDType,
     dtype_backend: DTypeBackend,
     implementation: Implementation,
-    backend_version: tuple[int, ...],
     version: Version,
 ) -> str | PandasDtype:
-    if dtype_backend is not None and dtype_backend not in {"pyarrow", "numpy_nullable"}:
+    if dtype_backend not in {None, "pyarrow", "numpy_nullable"}:
         msg = f"Expected one of {{None, 'pyarrow', 'numpy_nullable'}}, got: '{dtype_backend}'"
         raise ValueError(msg)
     dtypes = version.dtypes
-    if isinstance_or_issubclass(dtype, dtypes.Decimal):
-        msg = "Casting to Decimal is not supported yet."
-        raise NotImplementedError(msg)
-    if isinstance_or_issubclass(dtype, dtypes.Float64):
-        if dtype_backend == "pyarrow":
-            return "Float64[pyarrow]"
-        elif dtype_backend == "numpy_nullable":
-            return "Float64"
-        return "float64"
-    if isinstance_or_issubclass(dtype, dtypes.Float32):
-        if dtype_backend == "pyarrow":
-            return "Float32[pyarrow]"
-        elif dtype_backend == "numpy_nullable":
-            return "Float32"
-        return "float32"
-    if isinstance_or_issubclass(dtype, dtypes.Int64):
-        if dtype_backend == "pyarrow":
-            return "Int64[pyarrow]"
-        elif dtype_backend == "numpy_nullable":
-            return "Int64"
-        return "int64"
-    if isinstance_or_issubclass(dtype, dtypes.Int32):
-        if dtype_backend == "pyarrow":
-            return "Int32[pyarrow]"
-        elif dtype_backend == "numpy_nullable":
-            return "Int32"
-        return "int32"
-    if isinstance_or_issubclass(dtype, dtypes.Int16):
-        if dtype_backend == "pyarrow":
-            return "Int16[pyarrow]"
-        elif dtype_backend == "numpy_nullable":
-            return "Int16"
-        return "int16"
-    if isinstance_or_issubclass(dtype, dtypes.Int8):
-        if dtype_backend == "pyarrow":
-            return "Int8[pyarrow]"
-        elif dtype_backend == "numpy_nullable":
-            return "Int8"
-        return "int8"
-    if isinstance_or_issubclass(dtype, dtypes.UInt64):
-        if dtype_backend == "pyarrow":
-            return "UInt64[pyarrow]"
-        elif dtype_backend == "numpy_nullable":
-            return "UInt64"
-        return "uint64"
-    if isinstance_or_issubclass(dtype, dtypes.UInt32):
-        if dtype_backend == "pyarrow":
-            return "UInt32[pyarrow]"
-        elif dtype_backend == "numpy_nullable":
-            return "UInt32"
-        return "uint32"
-    if isinstance_or_issubclass(dtype, dtypes.UInt16):
-        if dtype_backend == "pyarrow":
-            return "UInt16[pyarrow]"
-        elif dtype_backend == "numpy_nullable":
-            return "UInt16"
-        return "uint16"
-    if isinstance_or_issubclass(dtype, dtypes.UInt8):
-        if dtype_backend == "pyarrow":
-            return "UInt8[pyarrow]"
-        elif dtype_backend == "numpy_nullable":
-            return "UInt8"
-        return "uint8"
-    if isinstance_or_issubclass(dtype, dtypes.String):
-        if dtype_backend == "pyarrow":
-            return "string[pyarrow]"
-        elif dtype_backend == "numpy_nullable":
-            return "string"
-        return str
-    if isinstance_or_issubclass(dtype, dtypes.Boolean):
-        if dtype_backend == "pyarrow":
-            return "boolean[pyarrow]"
-        elif dtype_backend == "numpy_nullable":
-            return "boolean"
-        return "bool"
-    if isinstance_or_issubclass(dtype, dtypes.Categorical):
-        # TODO(Unassigned): is there no pyarrow-backed categorical?
-        # or at least, convert_dtypes(dtype_backend='pyarrow') doesn't
-        # convert to it?
-        return "category"
+    base_type = dtype.base_type()
+    if pd_type := NW_TO_PD_DTYPES_INVARIANT.get(base_type):
+        return pd_type
+    if into_pd_type := NW_TO_PD_DTYPES_BACKEND.get(base_type):
+        return into_pd_type[dtype_backend]
     if isinstance_or_issubclass(dtype, dtypes.Datetime):
-        # Pandas does not support "ms" or "us" time units before version 2.0
-        if implementation is Implementation.PANDAS and backend_version < (
+        if is_pandas_or_modin(implementation) and PANDAS_VERSION < (
             2,
         ):  # pragma: no cover
+            if isinstance(dtype, dtypes.Datetime) and dtype.time_unit != "ns":
+                found = requires._unparse_version(PANDAS_VERSION)
+                available = f"available in 'pandas>=2.0', found version {found!r}."
+                changelog_url = "https://pandas.pydata.org/docs/dev/whatsnew/v2.0.0.html#construction-with-datetime64-or-timedelta64-dtype-with-unsupported-resolution"
+                msg = (
+                    f"`nw.Datetime(time_unit={dtype.time_unit!r})` is only {available}\n"
+                    "Narwhals has fallen back to using `time_unit='ns'` to avoid an error.\n\n"
+                    "Hint: to avoid this warning, consider either:\n"
+                    f"- Upgrading pandas: {changelog_url}\n"
+                    f"- Using a bare `nw.Datetime`, if this precision is not important"
+                )
+                issue_warning(msg, UserWarning)
             dt_time_unit = "ns"
         else:
             dt_time_unit = dtype.time_unit
@@ -475,11 +517,10 @@ def narwhals_to_native_dtype(  # noqa: C901, PLR0912, PLR0915
         if dtype_backend == "pyarrow":
             tz_part = f", tz={tz}" if (tz := dtype.time_zone) else ""
             return f"timestamp[{dt_time_unit}{tz_part}][pyarrow]"
-        else:
-            tz_part = f", {tz}" if (tz := dtype.time_zone) else ""
-            return f"datetime64[{dt_time_unit}{tz_part}]"
+        tz_part = f", {tz}" if (tz := dtype.time_zone) else ""
+        return f"datetime64[{dt_time_unit}{tz_part}]"
     if isinstance_or_issubclass(dtype, dtypes.Duration):
-        if implementation is Implementation.PANDAS and backend_version < (
+        if is_pandas_or_modin(implementation) and PANDAS_VERSION < (
             2,
         ):  # pragma: no cover
             du_time_unit = "ns"
@@ -492,9 +533,11 @@ def narwhals_to_native_dtype(  # noqa: C901, PLR0912, PLR0915
         )
     if isinstance_or_issubclass(dtype, dtypes.Date):
         try:
-            import pyarrow as pa  # ignore-banned-import
-        except ModuleNotFoundError:  # pragma: no cover
-            msg = "'pyarrow>=11.0.0' is required for `Date` dtype."
+            import pyarrow as pa  # ignore-banned-import  # noqa: F401
+        except ModuleNotFoundError as exc:  # pragma: no cover
+            # BUG: Never re-raised?
+            msg = "'pyarrow>=13.0.0' is required for `Date` dtype."
+            raise ModuleNotFoundError(msg) from exc
         return "date32[pyarrow]"
     if isinstance_or_issubclass(dtype, dtypes.Enum):
         if version is Version.V1:
@@ -505,32 +548,36 @@ def narwhals_to_native_dtype(  # noqa: C901, PLR0912, PLR0915
             return ns.CategoricalDtype(dtype.categories, ordered=True)
         msg = "Can not cast / initialize Enum without categories present"
         raise ValueError(msg)
-
-    if isinstance_or_issubclass(
-        dtype, (dtypes.Struct, dtypes.Array, dtypes.List, dtypes.Time, dtypes.Binary)
+    if issubclass(
+        base_type, (dtypes.Struct, dtypes.Array, dtypes.List, dtypes.Time, dtypes.Binary)
     ):
-        if implementation is Implementation.PANDAS and backend_version >= (2, 2):
-            try:
-                import pandas as pd
-                import pyarrow as pa  # ignore-banned-import  # noqa: F401
-            except ImportError as exc:  # pragma: no cover
-                msg = f"Unable to convert to {dtype} to to the following exception: {exc.msg}"
-                raise ImportError(msg) from exc
-            from narwhals._arrow.utils import (
-                narwhals_to_native_dtype as arrow_narwhals_to_native_dtype,
-            )
-
-            return pd.ArrowDtype(arrow_narwhals_to_native_dtype(dtype, version=version))
-        else:  # pragma: no cover
-            msg = (
-                f"Converting to {dtype} dtype is not supported for implementation "
-                f"{implementation} and version {version}."
-            )
-            raise NotImplementedError(msg)
+        return narwhals_to_native_arrow_dtype(dtype, implementation, version)
+    if issubclass(base_type, UNSUPPORTED_DTYPES):
+        msg = f"Converting to {base_type.__name__} dtype is not supported for {implementation}."
+        raise NotImplementedError(msg)
     msg = f"Unknown dtype: {dtype}"  # pragma: no cover
     raise AssertionError(msg)
 
 
+def narwhals_to_native_arrow_dtype(
+    dtype: IntoDType, implementation: Implementation, version: Version
+) -> pd.ArrowDtype:
+    if is_pandas_or_modin(implementation) and PANDAS_VERSION >= (2, 2):
+        try:
+            import pyarrow as pa  # ignore-banned-import  # noqa: F401
+        except ImportError as exc:  # pragma: no cover
+            msg = f"Unable to convert to {dtype} to to the following exception: {exc.msg}"
+            raise ImportError(msg) from exc
+        from narwhals._arrow.utils import narwhals_to_native_dtype as _to_arrow_dtype
+
+        return pd.ArrowDtype(_to_arrow_dtype(dtype, version))
+    msg = (  # pragma: no cover
+        f"Converting to {dtype} dtype is not supported for implementation "
+        f"{implementation} and version {version}."
+    )
+    raise NotImplementedError(msg)
+
+
 def int_dtype_mapper(dtype: Any) -> str:
     if "pyarrow" in str(dtype):
         return "Int64[pyarrow]"
@@ -539,58 +586,51 @@ def int_dtype_mapper(dtype: Any) -> str:
     return "int64"
 
 
-def calculate_timestamp_datetime(  # noqa: C901, PLR0912
-    s: NativeSeriesT, original_time_unit: str, time_unit: str
+_TIMESTAMP_DATETIME_OP_FACTOR: Mapping[
+    tuple[UnitCurrent, UnitTarget], tuple[BinOpBroadcast, IntoRhs]
+] = {
+    ("ns", "us"): (operator.floordiv, 1_000),
+    ("ns", "ms"): (operator.floordiv, 1_000_000),
+    ("us", "ns"): (operator.mul, NS_PER_MICROSECOND),
+    ("us", "ms"): (operator.floordiv, 1_000),
+    ("ms", "ns"): (operator.mul, NS_PER_MILLISECOND),
+    ("ms", "us"): (operator.mul, 1_000),
+    ("s", "ns"): (operator.mul, NS_PER_SECOND),
+    ("s", "us"): (operator.mul, US_PER_SECOND),
+    ("s", "ms"): (operator.mul, MS_PER_SECOND),
+}
+
+
+def calculate_timestamp_datetime(
+    s: NativeSeriesT, current: TimeUnit, time_unit: TimeUnit
 ) -> NativeSeriesT:
-    if original_time_unit == "ns":
-        if time_unit == "ns":
-            result = s
-        elif time_unit == "us":
-            result = s // 1_000
-        else:
-            result = s // 1_000_000
-    elif original_time_unit == "us":
-        if time_unit == "ns":
-            result = s * NS_PER_MICROSECOND
-        elif time_unit == "us":
-            result = s
-        else:
-            result = s // 1_000
-    elif original_time_unit == "ms":
-        if time_unit == "ns":
-            result = s * NS_PER_MILLISECOND
-        elif time_unit == "us":
-            result = s * 1_000
-        else:
-            result = s
-    elif original_time_unit == "s":
-        if time_unit == "ns":
-            result = s * NS_PER_SECOND
-        elif time_unit == "us":
-            result = s * US_PER_SECOND
-        else:
-            result = s * MS_PER_SECOND
-    else:  # pragma: no cover
-        msg = f"unexpected time unit {original_time_unit}, please report a bug at https://github.com/narwhals-dev/narwhals"
-        raise AssertionError(msg)
-    return result
+    if current == time_unit:
+        return s
+    if item := _TIMESTAMP_DATETIME_OP_FACTOR.get((current, time_unit)):
+        fn, factor = item
+        return fn(s, factor)
+    msg = (  # pragma: no cover
+        f"unexpected time unit {current}, please report an issue at "
+        "https://github.com/narwhals-dev/narwhals"
+    )
+    raise AssertionError(msg)
 
 
-def calculate_timestamp_date(s: NativeSeriesT, time_unit: str) -> NativeSeriesT:
-    s = s * SECONDS_PER_DAY
-    if time_unit == "ns":
-        result = s * NS_PER_SECOND
-    elif time_unit == "us":
-        result = s * US_PER_SECOND
-    else:
-        result = s * MS_PER_SECOND
-    return result
+_TIMESTAMP_DATE_FACTOR: Mapping[TimeUnit, int] = {
+    "ns": NS_PER_SECOND,
+    "us": US_PER_SECOND,
+    "ms": MS_PER_SECOND,
+    "s": 1,
+}
+
+
+def calculate_timestamp_date(s: NativeSeriesT, time_unit: TimeUnit) -> NativeSeriesT:
+    return s * SECONDS_PER_DAY * _TIMESTAMP_DATE_FACTOR[time_unit]
 
 
 def select_columns_by_name(
     df: NativeDataFrameT,
     column_names: list[str] | _1DArray,  # NOTE: Cannot be a tuple!
-    backend_version: tuple[int, ...],
     implementation: Implementation,
 ) -> NativeDataFrameT | Any:
     """Select columns by name.
@@ -601,7 +641,8 @@ def select_columns_by_name(
     if len(column_names) == df.shape[1] and (df.columns == column_names).all():
         return df
     if (df.columns.dtype.kind == "b") or (
-        implementation is Implementation.PANDAS and backend_version < (1, 5)
+        implementation is Implementation.PANDAS
+        and implementation._backend_version() < (1, 5)
     ):
         # See https://github.com/narwhals-dev/narwhals/issues/1349#issuecomment-2470118122
         # for why we need this
@@ -631,24 +672,12 @@ def import_array_module(implementation: Implementation, /) -> ModuleType:
         import numpy as np
 
         return np
-    elif implementation is Implementation.CUDF:
+    if implementation is Implementation.CUDF:
         import cupy as cp  # ignore-banned-import  # cuDF dependency.
 
         return cp
-    else:  # pragma: no cover
-        msg = f"Expected pandas/modin/cudf, got: {implementation}"
-        raise AssertionError(msg)
+    msg = f"Expected pandas/modin/cudf, got: {implementation}"  # pragma: no cover
+    raise AssertionError(msg)
 
 
-class PandasLikeSeriesNamespace(EagerSeriesNamespace["PandasLikeSeries", Any]):
-    @property
-    def implementation(self) -> Implementation:
-        return self.compliant._implementation
-
-    @property
-    def backend_version(self) -> tuple[int, ...]:
-        return self.compliant._backend_version
-
-    @property
-    def version(self) -> Version:
-        return self.compliant._version
+class PandasLikeSeriesNamespace(EagerSeriesNamespace["PandasLikeSeries", Any]): ...
